# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: game.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'game.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import location_pb2 as location__pb2
from . import spawn_pb2 as spawn__pb2
from . import team_pb2 as team__pb2
from . import turn_pb2 as turn__pb2
from . import world_pb2 as world__pb2
from . import world_object_pb2 as world__object__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\ngame.proto\x12\x05\x61\x65gis\x1a\x0elocation.proto\x1a\x0bspawn.proto\x1a\nteam.proto\x1a\nturn.proto\x1a\x0bworld.proto\x1a\x12world_object.proto\"[\n\tDroneScan\x12!\n\x08location\x18\x01 \x01(\x0b\x32\x0f.aegis.Location\x12\x19\n\x04team\x18\x02 \x01(\x0e\x32\x0b.aegis.Team\x12\x10\n\x08\x64uration\x18\x03 \x01(\x05\"8\n\x0f\x44roneScanUpdate\x12%\n\x0b\x64rone_scans\x18\x01 \x03(\x0b\x32\x10.aegis.DroneScan\"\x8b\x01\n\x14SurvivorHealthUpdate\x12!\n\x08location\x18\x01 \x01(\x0b\x32\x0f.aegis.Location\x12\x13\n\x0bsurvivor_id\x18\x02 \x01(\x05\x12\x12\n\nnew_health\x18\x03 \x01(\x05\x12\'\n\tnew_state\x18\x04 \x01(\x0e\x32\x14.aegis.SurvivorState\"\r\n\x0bGamesHeader\"W\n\nGameHeader\x12\x1b\n\x05world\x18\x01 \x01(\x0b\x32\x0c.aegis.World\x12\x0e\n\x06rounds\x18\x02 \x01(\x05\x12\x1c\n\x06spawns\x18\x03 \x03(\x0b\x32\x0c.aegis.Spawn\"\xf6\x01\n\x05Round\x12\r\n\x05round\x18\x01 \x01(\x05\x12\'\n\x0elayers_removed\x18\x02 \x03(\x0b\x32\x0f.aegis.Location\x12\x10\n\x08\x64\x65\x61\x64_ids\x18\x03 \x03(\x05\x12\x1a\n\x05turns\x18\x04 \x03(\x0b\x32\x0b.aegis.Turn\x12\"\n\tteam_info\x18\x05 \x03(\x0b\x32\x0f.aegis.TeamInfo\x12%\n\x0b\x64rone_scans\x18\x06 \x03(\x0b\x32\x10.aegis.DroneScan\x12<\n\x17survivor_health_updates\x18\x07 \x03(\x0b\x32\x1b.aegis.SurvivorHealthUpdate\"\x0c\n\nGameFooter\"\r\n\x0bGamesFooterb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'game_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_DRONESCAN']._serialized_start=107
  _globals['_DRONESCAN']._serialized_end=198
  _globals['_DRONESCANUPDATE']._serialized_start=200
  _globals['_DRONESCANUPDATE']._serialized_end=256
  _globals['_SURVIVORHEALTHUPDATE']._serialized_start=259
  _globals['_SURVIVORHEALTHUPDATE']._serialized_end=398
  _globals['_GAMESHEADER']._serialized_start=400
  _globals['_GAMESHEADER']._serialized_end=413
  _globals['_GAMEHEADER']._serialized_start=415
  _globals['_GAMEHEADER']._serialized_end=502
  _globals['_ROUND']._serialized_start=505
  _globals['_ROUND']._serialized_end=751
  _globals['_GAMEFOOTER']._serialized_start=753
  _globals['_GAMEFOOTER']._serialized_end=765
  _globals['_GAMESFOOTER']._serialized_start=767
  _globals['_GAMESFOOTER']._serialized_end=780
# @@protoc_insertion_point(module_scope)
