# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: event.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'event.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import game_pb2 as game__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0b\x65vent.proto\x12\x05\x61\x65gis\x1a\ngame.proto\"\x90\x02\n\x05\x45vent\x12*\n\x0cgames_header\x18\x01 \x01(\x0b\x32\x12.aegis.GamesHeaderH\x00\x12(\n\x0bgame_header\x18\x02 \x01(\x0b\x32\x11.aegis.GameHeaderH\x00\x12\x1d\n\x05round\x18\x03 \x01(\x0b\x32\x0c.aegis.RoundH\x00\x12(\n\x0bgame_footer\x18\x04 \x01(\x0b\x32\x11.aegis.GameFooterH\x00\x12*\n\x0cgames_footer\x18\x05 \x01(\x0b\x32\x12.aegis.GamesFooterH\x00\x12\x33\n\x11\x64rone_scan_update\x18\x06 \x01(\x0b\x32\x16.aegis.DroneScanUpdateH\x00\x42\x07\n\x05\x65ventb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'event_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_EVENT']._serialized_start=35
  _globals['_EVENT']._serialized_end=307
# @@protoc_insertion_point(module_scope)
