# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: world_object.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'world_object.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12world_object.proto\x12\x05\x61\x65gis\"K\n\x08Survivor\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0e\n\x06health\x18\x02 \x01(\x05\x12#\n\x05state\x18\x03 \x01(\x0e\x32\x14.aegis.SurvivorState\"F\n\x06Rubble\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x17\n\x0f\x65nergy_required\x18\x02 \x01(\x05\x12\x17\n\x0f\x61gents_required\x18\x03 \x01(\x05\"]\n\x0bWorldObject\x12#\n\x08survivor\x18\x01 \x01(\x0b\x32\x0f.aegis.SurvivorH\x00\x12\x1f\n\x06rubble\x18\x02 \x01(\x0b\x32\r.aegis.RubbleH\x00\x42\x08\n\x06object*$\n\rSurvivorState\x12\t\n\x05\x41LIVE\x10\x00\x12\x08\n\x04\x44\x45\x41\x44\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'world_object_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SURVIVORSTATE']._serialized_start=273
  _globals['_SURVIVORSTATE']._serialized_end=309
  _globals['_SURVIVOR']._serialized_start=29
  _globals['_SURVIVOR']._serialized_end=104
  _globals['_RUBBLE']._serialized_start=106
  _globals['_RUBBLE']._serialized_end=176
  _globals['_WORLDOBJECT']._serialized_start=178
  _globals['_WORLDOBJECT']._serialized_end=271
# @@protoc_insertion_point(module_scope)
